import { create_level } from "./core.mjs";

import bounding_walls from "./templates/bounding_walls.mjs";
import generate_oscillator from "./templates/oscillator.mjs";

function generate_corridor (length, data) {
	const result = [];

	const { pos, size, offset, spacing } = data;

	for (let n = 0; n < length; n++) {
		result.push({
			pos: [pos.x + n*(size.x + spacing), pos.y],
			size: [size.x, size.y],
			behaviour: "sine_y",
			meta: [
				["movement_scale", 85],
				["movement_speed", 1/2],
				["offset", offset.start + n * offset.step],
			],
		});
	}

	return result;
}



create_level(4, {
	player: {
		pos: {
			x: 20,
			y: 20,
		},
		speed: 80,
	},
	goal: {
		pos: {
			x: 20,
			y: 180,
		},
		size: {
			x: 30,
			y: 30,
		},
	},
	next_level: 5,
	enemies: [
		...bounding_walls(10),

		// top corridor
		// floor
		{
			pos: [0, 110],
			size: [600, 10],
		},
		// moving enemies section 1
		...generate_corridor(10, {
			pos: {
				x: 100,
				y: 95,
			},
			size: {
				x: 20,
				y: 40,
			},
			offset: {
				start: -1/4,
				step: -1/4,
			},
			spacing: 10,
		}),
		// moving enemies section 2
		...generate_oscillator(3, {
			pos: {
				x: 500,
				y: 20,
			},
			size: {
				x: 10,
				y: 10,
			},
			offset: 0,
			spacing: 25,
			meta: {
				movement_scale: 60,
				movement_speed: 1/4,
			},
		}),
		// corridor exit
		{
			pos: [690, 40],
			size: [10, 150],
			behaviour: "sine_x",
			meta: [
				["movement_scale", 90],
				["movement_speed", 5/8],
				["offset", 0],
			],
		},

		// second corridor
		// floor
		{
			pos: [100, 270],
			size: [700, 10],
		},

		...generate_oscillator(4, {
			pos: {
				x: 500,
				y: 135,
			},
			size: {
				x: 15,
				y: 15,
			},
			offset: 0,
			spacing: 20,
			meta: {
				movement_scale: 60,
				movement_speed: 1/4,
			},
		}),

		{
			pos: [255 - 20, 230],
			size: [20, 40],
			behaviour: "sine_x",
			meta: [
				["movement_scale", 155 - 20],
				["movement_speed", 1/12],
				["offset", 0],
			],
		},

		// bottom of level
		{
			pos: [0, 280],
			size: [800, 520],
		},
	],
});
